#include <Wire.h>
#define I2C_DEV_ADDR 0x55
#define LED_BUILTIN 2

volatile int8_t pwmVals[2] = {0, 0};
volatile bool newPWM = false;

const int pwm[]  = {19, 4};    // PWM pins for motors 3 and 4
const int in1[]  = {2, 16};
const int in2[]  = {15, 17};

void IRAM_ATTR onReceive(int len) {
  if (len >= 2) {
    pwmVals[0] = (int8_t)Wire.read();
    pwmVals[1] = (int8_t)Wire.read();
    newPWM = true;
  }
}

void setMotor(int dir, int pwmVal, int pwmPin, int in1Pin, int in2Pin) {
  analogWrite(pwmPin, pwmVal);
  digitalWrite(in1Pin, dir == 1 ? HIGH : LOW);
  digitalWrite(in2Pin, dir == -1 ? HIGH : LOW);
}

void setup() {
  Serial.begin(115200);
  Wire.onReceive(onReceive);
  Wire.begin((uint8_t)I2C_DEV_ADDR); // follower I2C
  pinMode(LED_BUILTIN, OUTPUT);
  for (int k = 0; k < 2; k++) {
    pinMode(pwm[k], OUTPUT);
    pinMode(in1[k], OUTPUT);
    pinMode(in2[k], OUTPUT);
  }
  Serial.println("Follower ready.");
}

void loop() {
  static int8_t pwmCopy[2] = {0, 0};

  if (newPWM) {
    noInterrupts();
    pwmCopy[0] = pwmVals[0];
    pwmCopy[1] = pwmVals[1];
    newPWM = false;
    interrupts();
  }

  for (int i = 0; i < 2; i++) {
    int dir = (i == 0) ? (pwmCopy[i] >= 0 ? -1 : 1) : (pwmCopy[i] >= 0 ? 1 : -1);
    int pwr = abs(pwmCopy[i]);
    setMotor(dir, pwr, pwm[i], in1[i], in2[i]);
  }

  digitalWrite(LED_BUILTIN, millis() % 1000 < 500 ? HIGH : LOW);
  delay(10);
}
